# -*- encoding : utf-8 -*-
class Event < ActiveRecord::Base
  AGE_LIMIT = %w(eighteen twenty none)
  EVENT_TYPE = %w(art concert course dj excenteraften football_match happening
                  luka_event meeting movie music performance quiz
                  samfundet_meeting show theater theme_party uka_event)
  STATUS = %w(active archived canceled)
  PRICE_TYPE = %w(included custom billig)

  TICKETS_INFORMATION_NAME = "billetter"

  attr_accessible :area_id, :billig_event_id, :title, :start_time, :end_time,
                  :age_limit, :organizer_id, :organizer_type,
                  :short_description, :long_description, :publication_time,
                  :spotify_uri, :facebook_link, :youtube_link, :spotify_link, 
                  :soundcloud_link, :instagram_link, :twitter_link, :lastfm_link, 
                  :snapchat_link, :vimeo_link, :general_link, :event_type, :status,
                  :primary_color, :secondary_color, :image_id,
                  :price_groups, :tmp_price_groups, :price_type

  validates_presence_of :title, :start_time, :end_time, :age_limit,
                        :event_type, :status, :area, :organizer, :price_type
  validates :start_time, date: { after: proc { Time.zone.now } }
  validates :end_time, date: { after: :start_time }
  validates :billig_event_id, uniqueness: true, allow_nil: true
  validates_inclusion_of :age_limit, in: AGE_LIMIT, message: "Invalid age limit"
  validates_inclusion_of :event_type, in: EVENT_TYPE, message: "Invalid type"
  validates_inclusion_of :status, in: STATUS, message: "Invalid status"
  validates_inclusion_of :price_type, in: PRICE_TYPE, message: "Invalid price type"
  validates_inclusion_of :organizer_type, in: [Group.name, ExternalOrganizer.name], message: "Invalid organizer type"

  validates :primary_color, css_hex_color: true
  validates :secondary_color, css_hex_color: true

  belongs_to :area
  belongs_to :organizer, polymorphic: true
  belongs_to :billig_event
  belongs_to :image
  has_one :front_page_lock
  has_many :price_groups, uniq: true

  before_save :enforce_price_choice

  default_scope includes(:area, :billig_event)

  scope :active, -> { where(status: 'active')}
  scope :published, -> { where("publication_time < ?", DateTime.current) }
  scope :upcoming, -> { where("start_time >= ?", Date.current) }
  scope :past, -> { where("start_time < ?", Date.current) }
  scope :today, -> {
    active
    .published
    .where(
      start_time:
        DateTime.current.change(hour: 4)..1.days.from_now.change(hour: 4))
  }
  scope :by_frontpage_weight, -> {
    active
    .published
    .upcoming
    .sort{|a,b| b.front_page_weight <=> a.front_page_weight }}

  include PgSearch
  pg_search_scope :search,
    against: [:title,
              :short_description,
              :long_description,
              :age_limit,
              :event_type],
    using: { tsearch: { dictionary: "english" } },
    associated_against: { area: :name }

  # Uses the above defined PgSearch scope to perform search.
  def self.text_search query
    if query.present?
      published.search(query)
    else
      []
    end
  end

  def to_s
    title
  end

  def image_or_default
    if image.present?
      image.image_file.url
    else
      Image.default_image.image_file.url
    end
  end

  def revealed_and_upcoming?
    status == 'active' &&
      publication_time < DateTime.current &&
      start_time >= Date.current
  end

  def currently_purchaseable?
    billig_event.present?
  end

  # This function calculates a weight for each event for placement on
  # the frontpage based on time until the event, the event type and
  # the area for the event.
  def front_page_weight
    weight = 50-4**((start_time - Time.current) / 1.day).to_i

    case event_type
    when "concert"
      weight += 7
    when "theme_party" 
      weight += 9
    when "football_match" 
      weight += -10000000
    when "dj"
      weight += -10000000
    else
      weight += 0
    end

    case area.name
    when "Storsalen"
      weight += 5
    else
      weight += 0
    end

    return weight
  end

  def to_param
    if title.nil?
      super
    else
      "#{id}-#{title.parameterize}"
    end
  end

  def organizer_group_id
    return organizer.id if organizer.is_a? Group
  end

  def organizer_external_name
    return organizer.name if organizer.is_a? ExternalOrganizer
  end

  def billig_price_groups
    # Samfundet3 only utilizes first ticket group.
    billig_event.billig_ticket_groups.first.billig_price_groups
  end

  def price
    case price_type
    when 'included'
      nil
    when 'custom'
      price_groups
    when 'billig'
      billig_price_groups
    end
  end

  def self.front_page_events n_front_page_events
    locks = FrontPageLock.locks_enabled
    locks_map = Hash[locks.map {|l| [l.position, l.event]}]

    events = Event.active
    .published
    .upcoming
    .includes(:area)
    .sort_by(&:front_page_weight)
    .reverse
    .take(n_front_page_events)

    front_page_events = []
    (0..n_front_page_events).map do |position|
      locked_event = locks_map[position]
      if locked_event
        front_page_events << locked_event
        events -= [locked_event]
      else
        front_page_events << nil
      end
    end

    front_page_events.map! do |event|
      event.nil? ? events.shift : event
    end

    front_page_events.compact
  end

  def self.tickets_information_page
    Page.find_or_create_by_name_en(name_no: TICKETS_INFORMATION_NAME,
      name_en: TICKETS_INFORMATION_NAME,
      role_id: Role.super_user.id)
  end

  # Introducing ghetto polymorphy. Should probably be done differently.
  def enforce_price_choice
    case price_type
    when 'included'
      self.price_groups.clear
      self.billig_event = nil
    when 'custom'
      self.billig_event = nil
    when 'billig'
      self.price_groups.clear
    end
  end

  def tmp_price_groups=(price_group_string)
    new_price_groups = price_group_string.split(",").map do |line|
      # Consume invalid price groups
      unless line =~ /(\w+\:\s*\d+)/i
        next
      end

      tmp=line.split(":")
      attrs = { name: tmp[0].strip, price: tmp[1].strip}

      existing_price_group = PriceGroup.where(attrs).first
      if existing_price_group.present?
        existing_price_group
      else
        PriceGroup.create!(attrs)
      end
    end

    self.price_groups = new_price_groups
  end

  def tmp_price_groups
    self.price_groups.uniq.map { |p| "#{p.name}: #{p.price}"} .join(", ")
  end

end

# == Schema Information
# Schema version: 20130422173230
#
# Table name: events
#
#  id                :integer          not null, primary key
#  title             :string(255)
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#  start_time        :datetime
#  end_time          :datetime
#  short_description :text
#  long_description  :text
#  organizer_id      :integer
#  organizer_name    :string(255)
#  area_id           :integer
#  publication_time  :datetime
#  age_limit         :string(10)
#  spotify_uri       :string(255)
#  event_type        :string(255)
#  status            :string(255)
#

